#encoding=utf-8
class LimitStatement(object):
    """
    Crea la sentencia LIMIT
    """ 
    def __init__(self, n):
        super(LimitStatement, self).__init__()
        self.n = n
    def result(self):
        """
        Construye la expresion
        """ 
        return "LIMIT %s" % (self.n)

class SkipStatement(object):
    """
    Crea la sentencia SKIP (similar a OFFSET)
    """ 
    def __init__(self, n):
        super(SkipStatement, self).__init__()
        self.n = n
    def result(self):
        """
        Construye la expresion
        """ 
        return "SKIP %s" % (self.n)

class FetchPlanStatement(object):
    """
    Crea la sentencia FETCHPLAN, usada en OrientDB para definir la estrategia de obtención de recursos.
    """ 
    def __init__(self, direction='*', depth=-1):
        super(FetchPlanStatement, self).__init__()
        self.direction = direction
        self.depth = depth

    def result(self):
        """
        Construye la expresion
        """ 
        return "FETCHPLAN %s:%d" % (self.direction, self.depth)



#encoding=utf-8
class BaseGrouper(object):
    """
    Clase base para las agrupaciones de clausulas principalmente usado para concatenar expresiones condicionales
    """
    def __init__(self, *args, **kwargs):
        super(BaseGrouper, self).__init__(*args, **kwargs)
        self.items = []
    def append(self, item):
        self.items.append(item)
    def size(self):
        return len(self.items)
    def result(self, concatenator = ' '):
        concatenator = str(concatenator)
        chunks = []
        for i in self.items:
            chunks.append(i.result())
        return concatenator.join( chunks )

# class ANDGrouping(BaseGrouper):
#     def __init__(self, *args, **kwargs):
#         super(ANDGrouping, self).__init__(*args, **kwargs)
#     def result(self):
#         return super(ANDGrouping, self).result(' AND ')

# class ORGrouping(BaseGrouper):
#     def __init__(self, *args, **kwargs):
#         super(ORGrouping, self).__init__(*args, **kwargs)
#     def result(self):
#         return super(ORGrouping, self).result(' OR ')



#encoding=utf-8
from . import statements
from . import grouping
from . import expressions
import json

class Select:
    """SELECT Query Builder"""
    def __init__(self, *fields):
        """
        Inicializa la consulta SELECT opcionalmente los argumentos pasados serna considerados campos para la proyección.
        """
        super(self.__class__, self).__init__()
        self.raw_fields = fields
        self.raw_fields = []
        self.raw_fields_group = []
        self.fields = []
        self.group_fields = []
        self.raw_tables = []
        self.raw_order_by = []
        self.order_by_fields = []
        self.tables = []
        self.where_criteria = grouping.BaseGrouper()

    def __prepareData__(self):
        """
        Helper para preparar los datos para la produccion del SQL final.
        """
        if isinstance(self.raw_fields, str):
            self.raw_fields = self.raw_fields.split(',')

        for x in self.raw_fields:
            self.fields.append(expressions.AliasExpression(x).result())
        
        if len(self.fields) == 0:
            self.fields.append('*')

        for x in self.raw_tables:
            self.tables.append(expressions.AliasExpression(x).result())

        if isinstance(self.raw_fields_group, str):
            self.raw_fields_group = self.raw_fields_group.split(',')


        for x in self.raw_fields_group:
            self.group_fields.append(expressions.AliasExpression(x).result())


        if isinstance(self.raw_order_by, str):
            self.raw_order_by = self.raw_order_by.split(',')

        for x in self.raw_order_by:
            self.order_by_fields.append(expressions.OrderByExpression(*x).result())

    def from_(self, table, alias=None):
        """
        Establece el origen de datos (y un alias opcionalmente).
        """
        if isinstance(table, str):
            table = [[table, alias]]
        self.raw_tables = table
        return self

    def where(self, field, value = None, operator = None):
        """
        Establece condiciones para la consulta unidas por AND
        """
        conjunction = None
        if value is None and isinstance(field, dict):
            for f,v in field.items():
                if self.where_criteria.size() > 0:
                    conjunction = 'AND'

                self.where_criteria.append(expressions.ConditionExpression(f, v, operator=operator, conjunction=conjunction))
                
        else:
            if self.where_criteria.size() > 0:
                    conjunction = 'AND'
            self.where_criteria.append(expressions.ConditionExpression(field, value, operator=operator, conjunction=conjunction))
        return self

    def where_or(self, field, value = None, operator = None):
        """
        Establece condiciones para la consulta unidas por OR
        """
        conjunction = None
        if value is None and isinstance(field, dict):
            for f,v in field.items():
                if self.where_criteria.size() > 0:
                    conjunction = 'OR'

                self.where_criteria.append(expressions.ConditionExpression(f, v, operator=operator, conjunction=conjunction))
                
        else:
            if self.where_criteria.size() > 0:
                    conjunction = 'OR'
            self.where_criteria.append(expressions.ConditionExpression(field, value, operator=operator, conjunction=conjunction))
        return self

    def group_by(self, *args):
        """
        Indica los campos para agrupación
        """
        if len(args) == 1:
            self.raw_fields_group = args[0].split(',')
        else:
            self.raw_fields_group = list(args)
        return self

    def order_by(self, field, orientation='ASC'):
        """
        Indica los campos y el criterio de ordenamiento
        """
        if isinstance(field, list):
            self.raw_order_by.append(field)
        else:
            self.raw_order_by.append([field, orientation])

        return self

    def result(self, *args, **kwargs):
        """
        Construye la consulta SQL
        """
        prettify = kwargs.get('pretty', False)
        self.__prepareData__()
        sql = 'SELECT '
        sql +=  ', '.join(self.fields)
        
        if len(self.tables) > 0:
            if prettify:
                sql += '\n'
            else:
                sql += ' '
            sql +=  'FROM '
            sql +=  ', '.join(self.tables)
        if self.where_criteria.size() > 0:
            if prettify:
                sql += '\n'
            else:
                sql += ' '
            sql +=  'WHERE '
            sql +=  self.where_criteria.result()
        if len(self.group_fields) > 0:
            if prettify:
                sql += '\n'
            else:
                sql += ' '
            sql +=  'GROUP BY '
            sql +=  ', '.join(self.group_fields)
        if len(self.order_by_fields) > 0:
            if prettify:
                sql += '\n'
            else:
                sql += ' '
            sql +=  'ORDER BY '
            sql +=  ', '.join(self.order_by_fields)
            if prettify:
                sql += '\n'
            else:
                sql += ' '
        
        return sql

class Delete(object):
    """
    DELETE Query Builder
    """

    def __init__(self, type):
        """
        Inicializa la consulta, type = recurso (Vertex|Edge)
        """
        super(Delete, self).__init__()
        self._class = None
        self._cluster = None
        self._type = None
        self.data = {}
        self.where_criteria = grouping.BaseGrouper()
        self._type = type
    
    def class_(self, _class):
        """
        Especifica la clase para eliminar
        """
        self._class = _class
        return self
        
    def where(self, field, value = None, operator = None):
        """
        Establece condiciones para la consulta unidas por AND
        """
        conjunction = None
        if value is None and isinstance(field, dict):
            for f,v in field.items():
                if self.where_criteria.size() > 0:
                    conjunction = 'AND'
                self.where_criteria.append(expressions.ConditionExpression(f, v, operator=operator, conjunction=conjunction))
                
        else:
            if self.where_criteria.size() > 0:
                    conjunction = 'AND'
            self.where_criteria.append(expressions.ConditionExpression(field, value, operator=operator, conjunction=conjunction))
        return self

    def where_or(self, field, value = None, operator = None):
        """
        Establece condiciones para la consulta unidas por OR
        """
        conjunction = None
        if value is None and isinstance(field, dict):
            for f,v in field.items():
                if self.where_criteria.size() > 0:
                    conjunction = 'OR'
                self.where_criteria.append(expressions.ConditionExpression(f, v, operator=operator, conjunction=conjunction))
                
        else:
            if self.where_criteria.size() > 0:
                    conjunction = 'OR'
            self.where_criteria.append(expressions.ConditionExpression(field, value, operator=operator, conjunction=conjunction))
        return self

    def result(self, *args, **kwargs):
        """
        Construye la consulta SQL
        """
        prettify = kwargs.get('pretty', False)

        sql = 'DELETE %s %s' % (self._type, self._class)
        
        if prettify:
            sql += '\n'
        else:
            sql += ' '

        if self.where_criteria.size() > 0:
            sql +=  'WHERE '
            sql +=  self.where_criteria.result()
            if prettify:
                sql += '\n'
            else:
                sql += ' '
        
        return sql

class Create(object):
    "CREATE query builder"

    def __init__(self, type):
        """
        Inicializa la consulta, type = recurso (Vertex|Edge)
        """
        super(Create, self).__init__()
        self._class = None
        self._cluster = None
        self._type = None
        self._from = None
        self._to = None
        self.data = {}
        self._type = type
    
    def class_(self, _class):
        """
        Especifica la clase para crear
        """
        self._class = _class
        return self
        
    def cluster(self, cluster):
        """
        Especifica el cluster donde se almacenara el nuevo recurso
        """
        self._cluster = cluster
        return self
    
    def from_(self, From):
        """
        [Edge-only] especifica el origen del lado
        """
        if self._type.lower() != 'edge':
            raise ValueError('Cannot set From/To to non-edge objects')
        self._from = From
        return self
    
    def to(self, to):
        """
        [Edge-only] especifica el destino del lado
        """
        if self._type.lower() != 'edge':
            raise ValueError('Cannot set From/To to non-edge objects')
        self._to = to
        return self

    def set(self, field, value = None):
        """
        [Edge|Vertex] establece datos del recurso
        """
        if value is None and isinstance(field, dict):
            self.content(field)
        if field and value:
            self.data[field] = value
        return self

    def content(self, obj):
        """
        [Edge|Vertex] establece datos del recurso
        """
        self.data.update(obj)
        return self

    def result(self, *args, **kwargs):
        """
        Construye la consulta SQL
        """
        prettify = kwargs.get('pretty', False)

        sql = 'CREATE %s %s' % (self._type, self._class)
        
        if prettify:
            sql += '\n'
        else:
            sql += ' '

        if self._type.lower() == 'edge':
            sql += " FROM %s TO %s " % (self._from, self._to)
        
        if self._cluster:
            sql += 'CLUSTER %s' % self._cluster
            if prettify:
                sql += '\n'
            else:
                sql += ' '
        
        if self.data:
            sql += 'CONTENT ' + json.dumps(self.data)
        print (sql)
        return sql

class Update(object):
    """
    UPDATE Query Builder
    """
    def __init__(self, _class):
        """
        Inicializa la clase, _class = origen donde actualizar
        """
        super(Update, self).__init__()
        self._class = None
        self._cluster = None
        self.data = {}
        self.where_criteria = grouping.BaseGrouper()
        self._class = _class

    def set(self, field, value = None):
        """
        [Edge|Vertex] establece datos del recurso
        """
        if value is None and isinstance(field, dict):
            self.content(field)
        if field and value:
            self.data[field] = value
        return self

    def content(self, obj):
        """
        [Edge|Vertex] establece datos del recurso
        """
        self.data.update(obj)
        return self

    def where(self, field, value = None, operator = None):
        """
        Establece condiciones para la consulta unidas por AND
        """
        conjunction = None
        if value is None and isinstance(field, dict):
            for f,v in field.items():
                if self.where_criteria.size() > 0:
                    conjunction = 'AND'
                self.where_criteria.append(expressions.ConditionExpression(f, v, operator=operator, conjunction=conjunction))
                
        else:
            if self.where_criteria.size() > 0:
                    conjunction = 'AND'
            self.where_criteria.append(expressions.ConditionExpression(field, value, operator=operator, conjunction=conjunction))
        return self

    def where_or(self, field, value = None, operator = None):
        """
        Establece condiciones para la consulta unidas por OR
        """
        conjunction = None
        if value is None and isinstance(field, dict):
            for f,v in field.items():
                if self.where_criteria.size() > 0:
                    conjunction = 'OR'
                self.where_criteria.append(expressions.ConditionExpression(f, v, operator=operator, conjunction=conjunction))
                
        else:
            if self.where_criteria.size() > 0:
                    conjunction = 'OR'
            self.where_criteria.append(expressions.ConditionExpression(field, value, operator=operator, conjunction=conjunction))
        return self

    def result(self, *args, **kwargs):
        """
        Construye la consulta SQL
        """
        prettify = kwargs.get('pretty', False)

        sql = 'UPDATE %s' % self._class
        
        if prettify:
            sql += '\n'
        else:
            sql += ' '
        
        if self.data:
            sql += 'MERGE ' + json.dumps(self.data)
            if prettify:
                sql += '\n'
            else:
                sql += ' '

        if self.where_criteria.size() > 0:
            sql +=  'WHERE '
            sql +=  self.where_criteria.result()
            if prettify:
                sql += '\n'
            else:
                sql += ' '
        
        print (sql)
        return sql



import pqb
from setuptools import setup

setup(name='pqb',
      version=pqb.__version__,
      description='Simple SQL Query Builder',
      url='http://github.com/josegomezr/pqb',
      author='Jose Gomez',
      author_email='1josegomezr@gmail.com',
      license='MIT',
      packages=['pqb'],
      zip_safe=False
    )


#encoding=utf-8
import re
REGEX_CLEANER = re.compile(r"[^a-z0-9_]", re.I)

class AliasExpression(object):
    """
    Crea una expresion SQL alias <campo> AS <alias> o simplemente <campo> si no fue provisto un alias
    """
    def __init__(self, field_name, alias=None, *args, **kwargs):
        super(self.__class__, self).__init__()
        if isinstance(field_name, list):
            field_name, alias = field_name
        if ' as ' in field_name.lower():
            self.field_name, self.alias = field_name.split(' as ')
        else:
            self.field_name = field_name
            self.alias = alias
    def result(self):
        """
        Construye la expresion
        """ 
        field = re.sub(REGEX_CLEANER, '', self.field_name)

        if self.alias:
            alias = re.sub(REGEX_CLEANER, '', self.alias)
            return "%s AS %s" % (field, alias)
        else:
            return field

class ConditionExpression(object):
    """
    Crea una expresion SQL condicional, usadas en WHERE y HAVING:
    <campo> <operador> <valor>
    """
    def __init__(self, field, value, *args, **kwargs):
        self.field = field
        self.value = value
        self.operator = '=' if kwargs.get('operator') is None else kwargs['operator']
        self.conjunction = kwargs.get('conjunction')

    def result(self):
        """
        Construye la expresion
        """ 
        field = re.sub(REGEX_CLEANER, '', self.field)

        try:
            value = float(self.value)
        except ValueError:
            value = str(self.value) \
                .replace("\\", r"\\") \
                .replace('"', r'\"') \
                .replace("'", r"\'")

            value = "'%s'" % value
        
        res = "%s %s %s" % (field, self.operator, value)

        if self.conjunction:
            res = "%s %s" % (self.conjunction, res)
        
        return res

class OrderByExpression(object):
    """
    Crea una expresion SQL de ordenamiento.
    """ 
    def __init__(self, field, orientation = 'ASC'):
        super(OrderByExpression, self).__init__()

        if isinstance(field, list):
            self.field, self.orientation = field[0:2]
        elif 'ASC' in field.upper() or 'DESC' in field.upper():
            self.field, self.orientation = field.split(' ')
        else:
            self.field = field
            self.orientation = orientation
    def result(self):
        """
        Construye la expresion
        """ 
        return "%s %s" % (self.field, self.orientation)